import itertools
import logging
import os
from numpy import random
import numpy as np

import cocotb
from cocotb.clock import Clock
from cocotb.triggers import RisingEdge, Timer, ClockCycles
from cocotb.regression import TestFactory

from cocotbext.ahb import AHBLiteMaster, AHBBus

ID = 0x4326BF00

async def SPI_RESET(dut,tb):
    await tb.config_ahb_master.write(0x08, 0x00000066)
    await tb.config_ahb_master.write(0x08, 0x00000166)

    data = await tb.config_ahb_master.read(0x04,4)
    while(int(data[0]['data'],16)&1):
        await ClockCycles(dut.HCLK,2)
        data = await tb.config_ahb_master.read(0x04,4)
    
    await tb.config_ahb_master.write(0x08, 0x00000099)
    await tb.config_ahb_master.write(0x08, 0x00000199)
    
    data = await tb.config_ahb_master.read(0x04,4)
    while(int(data[0]['data'],16)&1):
        await ClockCycles(dut.HCLK,2)
        data = await tb.config_ahb_master.read(0x04,4)
            
async def SPI_READ_JEDIC(dut, tb):
    await tb.config_ahb_master.write(0x08, 0x0002009F)
    await tb.config_ahb_master.write(0x08, 0x0002039F)
    data = await tb.config_ahb_master.read(0x04,4)
    while(int(data[0]['data'],16)&1):
        await ClockCycles(dut.HCLK,10)
        data = await tb.config_ahb_master.read(0x04,4)
    ID = await tb.config_ahb_master.read(0x10,4)
    return int(ID[0]['data'],16)

async def SET_QPI_MODE(dut,tb):
    await tb.config_ahb_master.write(0x08, 0x00000038)
    await tb.config_ahb_master.write(0x08, 0x00000138)

    data = await tb.config_ahb_master.read(0x04,4)
    while(int(data[0]['data'],16)&1):
        await ClockCycles(dut.HCLK,2)
        data = await tb.config_ahb_master.read(0x04,4)

async def LEAVE_QPI_MODE(dut, tb):
    await tb.config_ahb_master.write(0x08, 0x000000FF)
    await tb.config_ahb_master.write(0x08, 0x000001FF)

    data = await tb.config_ahb_master.read(0x04,4)
    while(int(data[0]['data'],16)&1):
        await ClockCycles(dut.HCLK,2)
        data = await tb.config_ahb_master.read(0x04,4)

async def QPI_READ_JEDIC(dut, tb):
    await tb.config_ahb_master.write(0x08, 0x000220AF)
    await tb.config_ahb_master.write(0x08, 0x000223AF)
    data = await tb.config_ahb_master.read(0x04,4)
    while(int(data[0]['data'],16)&1):
        await ClockCycles(dut.HCLK,10)
        data = await tb.config_ahb_master.read(0x04,4)
    ID = await tb.config_ahb_master.read(0x10,4)
    return int(ID[0]['data'],16) 
      
async def SPI_WRITE_ENABLE(dut, tb):
    await tb.config_ahb_master.write(0x08, 0x00000006)
    await tb.config_ahb_master.write(0x08, 0x00000106)

    data = await tb.config_ahb_master.read(0x04,4)
    while(int(data[0]['data'],16)&1):
        await ClockCycles(dut.HCLK,2)
        data = await tb.config_ahb_master.read(0x04,4)
 
async def SPI_WRITE_DISABLE(dut, tb):
    await tb.config_ahb_master.write(0x08, 0x00000004)
    await tb.config_ahb_master.write(0x08, 0x00000104)

    data = await tb.config_ahb_master.read(0x04,4)
    while(int(data[0]['data'],16)&1):
        await ClockCycles(dut.HCLK,2)
        data = await tb.config_ahb_master.read(0x04,4)
       
async def SPI_CLR_PROTECTION(dut, tb):
    await tb.config_ahb_master.write(0x08, 0x00000098)
    await tb.config_ahb_master.write(0x08, 0x00000198)

    data = await tb.config_ahb_master.read(0x04,4)
    while(int(data[0]['data'],16)&1):
        await ClockCycles(dut.HCLK,2)
        data = await tb.config_ahb_master.read(0x04,4)
   
async def SPI_CHIP_ERASE(dut, tb):
    await tb.config_ahb_master.write(0x08, 0x000000C7)
    await tb.config_ahb_master.write(0x08, 0x000001C7)

    data = await tb.config_ahb_master.read(0x04,4)
    while(int(data[0]['data'],16)&1):
        await ClockCycles(dut.HCLK,2)
        data = await tb.config_ahb_master.read(0x04,4)

async def SPI_BLOCK_ERASE(dut, tb):
    # Write Adress
    await tb.config_ahb_master.write(0x0C, 0x000000)
    
    await tb.config_ahb_master.write(0x08, 0x000008D8)
    await tb.config_ahb_master.write(0x08, 0x000009D8)

    data = await tb.config_ahb_master.read(0x04,4)
    while(int(data[0]['data'],16)&1):
        await ClockCycles(dut.HCLK,2)
        data = await tb.config_ahb_master.read(0x04,4)
    SPI_STAT = await SPI_READ_STAT_REG(dut, tb)
    while(SPI_STAT&(1<<24)):
        SPI_STAT = await SPI_READ_STAT_REG(dut, tb)
        await Timer(time=1, units='ms')
   
async def SPI_READ_STAT_REG(dut,tb):
    await tb.config_ahb_master.write(0x08, 0x00002005)
    await tb.config_ahb_master.write(0x08, 0x00002305)
    data = await tb.config_ahb_master.read(0x04,4)
    while(int(data[0]['data'],16)&1):
        await ClockCycles(dut.HCLK,10)
        data = await tb.config_ahb_master.read(0x04,4)
    data = await tb.config_ahb_master.read(0x10,4)
    return (int(data[0]['data'],16))

async def QPI_READ_STAT_REG(dut,tb):
    await tb.config_ahb_master.write(0x08, 0x00002005)
    await tb.config_ahb_master.write(0x08, 0x00002305)
    data = await tb.config_ahb_master.read(0x04,4)
    while(int(data[0]['data'],16)&1):
        await ClockCycles(dut.HCLK,10)
        data = await tb.config_ahb_master.read(0x04,4)
    data = await tb.config_ahb_master.read(0x10,4)
    return (int(data[0]['data'],16))


async def SPI_READ_SLOW(dut, tb):
    # Write Adress
    await tb.config_ahb_master.write(0x0C, 0x000000)
    
    await tb.config_ahb_master.write(0x08, 0x00001003)
    await tb.config_ahb_master.write(0x08, 0x00001303)
    data = await tb.config_ahb_master.read(0x04,4)
    while(int(data[0]['data'],16)&1):
        await ClockCycles(dut.HCLK,10)
        data = await tb.config_ahb_master.read(0x04,4)
    rData = await tb.config_ahb_master.read(0x10,4)
    print(rData)

async def QPI_READ_WORDS(dut,tb, addr =0, n_words=1, cont_read = False):
    # Write Adress
    await tb.config_ahb_master.write(0x0C, addr)

    if(cont_read==True):
        QSPI_CTRL_REG = await tb.config_ahb_master.read(0x00,4)
        QSPI_CTRL_REG = int(QSPI_CTRL_REG[0]['data'],16)
        QSPI_CTRL_REG = QSPI_CTRL_REG | ((0xA0<<16) + (1<<24))
        await tb.config_ahb_master.write(0x00, QSPI_CTRL_REG)
        QSPI_DUMMY_CYCLES = 4
    else:
        QSPI_DUMMY_CYCLES = 6


    QSPI_CMD  = 0x0B
    QSPI_READ = 1
    QSPI_WRITE = 0
    QSPI_ADDR_EN = 1
    QSPI_N_RW_BYTES = n_words*4 - 1
    
    QSPI_INSTR = QSPI_CMD
    QSPI_INSTR += QSPI_READ<<9
    QSPI_INSTR += QSPI_WRITE<<10
    QSPI_INSTR += QSPI_ADDR_EN<<11
    QSPI_INSTR += QSPI_DUMMY_CYCLES<<12
    QSPI_INSTR += QSPI_N_RW_BYTES << 16

    await tb.config_ahb_master.write(0x08, QSPI_INSTR)
    QSPI_INSTR = QSPI_INSTR ^ (1<<8)
    await tb.config_ahb_master.write(0x08, QSPI_INSTR)
    
    data = await tb.config_ahb_master.read(0x04,4)
    while(int(data[0]['data'],16)&1):
        await ClockCycles(dut.HCLK,10)
        data = await tb.config_ahb_master.read(0x04,4)
        
    apb_r_addrs=[0x10, 0x14, 0x18, 0x1C]
    rData = []
    for i in range(0,n_words):
        rData_temp = await tb.config_ahb_master.read(apb_r_addrs[i],4)
        rData.append(int(rData_temp[0]['data'],16))
    return rData
    
async def QPI_PAGE_PROGRAM(dut, tb, addr = 0, words=0, n_words = 1):
    # Write Adress
    await tb.config_ahb_master.write(0x0C, addr)
    # Write Data
    
    apb_addrs = [0x20, 0x24, 0x28, 0x2C]
    
    for i in range(0,n_words):
        await tb.config_ahb_master.write(apb_addrs[i], words[i])

    QSPI_CMD  = 0x02
    QSPI_READ = 0
    QSPI_WRITE = 1
    QSPI_ADDR_EN = 1
    QSPI_DUMMY_CYCLES = 0
    QSPI_N_RW_BYTES = n_words*4 - 1
    
    QSPI_INSTR = QSPI_CMD
    QSPI_INSTR += QSPI_READ<<9
    QSPI_INSTR += QSPI_WRITE<<10
    QSPI_INSTR += QSPI_ADDR_EN<<11
    QSPI_INSTR += QSPI_DUMMY_CYCLES<<12
    QSPI_INSTR += QSPI_N_RW_BYTES << 16

    await tb.config_ahb_master.write(0x08, QSPI_INSTR)
    QSPI_INSTR = QSPI_INSTR ^ (1<<8)
    await tb.config_ahb_master.write(0x08, QSPI_INSTR)
    
    data = await tb.config_ahb_master.read(0x04,4)
    while(int(data[0]['data'],16)&1):
        await ClockCycles(dut.HCLK,10)
        data = await tb.config_ahb_master.read(0x04,4)
    SPI_STAT = await QPI_READ_STAT_REG(dut, tb)
    while(SPI_STAT&(1<<24)):
        SPI_STAT = await QPI_READ_STAT_REG(dut, tb)
        await Timer(time=1, units='us')



